@@ -39,7 +39,6 @@
 #endif
 
 #include <string.h>
-#include "util.h"
 #include "relaylib.h"
 #include "srtypes.h"
 #include "http.h"
@@ -85,7 +84,6 @@ static void thread_accept(void *notused);
 static error_code try_port(u_short port, char *if_name, char *relay_ip);
 static void thread_send(void *notused);
 
-
 #define BUFSIZE (1024)
 
 #define HTTP_HEADER_TIMEOUT 2
@@ -116,11 +114,9 @@ tag_compare (char *str, char *tag)
     int i, a,b;
     int len;
 
-
     len = strlen(tag);
 
-    for (i=0; i<len; i++) 
-    {
+    for (i=0; i<len; i++) {
 	a = tolower (str[i]);
 	b = tolower (tag[i]);
 	if ((a != b) || (a == 0))
@@ -140,8 +136,7 @@ header_receive (int sock, int *icy_metadata)
 
     *icy_metadata = 0;
         
-    while (1)
-    {
+    while (1) {
 	// use select to prevent deadlock on malformed http header
 	// that lacks CRLF delimiter
 	FD_ZERO(&fds);
@@ -162,16 +157,14 @@ header_receive (int sock, int *icy_metadata)
 
 	buf[r] = 0;
 	md = strtok (buf, HTTP_HEADER_DELIM);
-	while (md)
-	{
+	while (md) {
 	    debug_printf ("Got token: %s\n", md);
 	    // Finished when we are at end of header: only CRLF will be there.
 	    if ((md[0] == '\r') && (md[1] == 0))
 		return 0;
 	
 	    // Check for desired tag
-	    if (tag_compare (md, ICY_METADATA_TAG) == 0) 
-	    {
+	    if (tag_compare (md, ICY_METADATA_TAG) == 0) {
 		for (md += strlen(ICY_METADATA_TAG); md[0] && (isdigit(md[0]) == 0); md++);
 		
 		if (md[0])
@@ -202,29 +195,24 @@ swallow_receive (int sock)
     BOOL hasmore = TRUE;
         
     FD_ZERO(&fds);
-    while(hasmore)
-    {
+    while (hasmore) {
         // Poll the socket to see if it has anything to read
         hasmore = FALSE;
         FD_SET(sock, &fds);
         tv.tv_sec = 0;
         tv.tv_usec = 0;
         ret = select(sock + 1, &fds, NULL, NULL, &tv);
-        if (ret == 1)
-        {
+        if (ret == 1) {
             // Read and throw away data, ignoring errors
             ret = recv(sock, buf, BUFSIZE, 0);
-            if (ret > 0)
-            {
+            if (ret > 0) {
                 hasmore = TRUE;
             }
-            else if (ret == SOCKET_ERROR)
-            {
+            else if (ret == SOCKET_ERROR) {
                 break;
             }
         }
-        else if (ret == SOCKET_ERROR)
-        {
+        else if (ret == SOCKET_ERROR) {
             break;
         }
     }
@@ -241,8 +229,7 @@ make_nonblocking (int sock)
 
 #ifndef WIN32
     opt = fcntl(sock, F_GETFL);
-    if (opt != SOCKET_ERROR)
-    {
+    if (opt != SOCKET_ERROR) {
         fcntl(sock, F_SETFL, opt | O_NONBLOCK);
     }
 #else
@@ -251,13 +238,11 @@ make_nonblocking (int sock)
 #endif
 }
 
-
 BOOL relaylib_isrunning()
 {
     return m_running;
 }
 
-
 error_code
 relaylib_set_response_header(char *http_header)
 {
@@ -280,20 +265,28 @@ catch_pipe(int code)
 #endif
 
 error_code
-relaylib_init (BOOL search_ports, int relay_port, int max_port, 
-	       int *port_used, char *if_name, int max_connections, 
+relaylib_init (BOOL search_ports, u_short relay_port, u_short max_port, 
+	       u_short *port_used, char *if_name, int max_connections, 
 	       char *relay_ip, int have_metadata)
 {
     int ret;
 #ifdef WIN32
     WSADATA wsd;
-
-    if (WSAStartup(MAKEWORD(2,2), &wsd) != 0)
-        return SR_ERROR_CANT_BIND_ON_PORT;
 #endif
 
-    if (relay_port < 1 || !port_used)
+    debug_printf ("relaylib_init()\n");
+
+#ifdef WIN32
+    if (WSAStartup(MAKEWORD(2,2), &wsd) != 0) {
+	debug_printf ("relaylib_init(): SR_ERROR_CANT_BIND_ON_PORT\n");
+        return SR_ERROR_CANT_BIND_ON_PORT;
+    }
+#endif
+
+    if (relay_port < 1 || !port_used) {
+	debug_printf ("relaylib_init(): SR_ERROR_INVALID_PARAM\n");
         return SR_ERROR_INVALID_PARAM;
+    }
 
 #ifndef WIN32
     // catch a SIGPIPE if send fails
@@ -317,13 +310,13 @@ relaylib_init (BOOL search_ports, int relay_port, int max_port,
         max_port = relay_port;
 
     for(;relay_port <= max_port; relay_port++) {
-        ret = try_port((u_short)relay_port, if_name, relay_ip);
+        ret = try_port ((u_short)relay_port, if_name, relay_ip);
         if (ret == SR_ERROR_CANT_BIND_ON_PORT)
             continue;           // Keep searching.
 
         if (ret == SR_SUCCESS) {
             *port_used = relay_port;
-            debug_printf("Relay: Listening on port %d\n", relay_port);
+            debug_printf ("Relay: Listening on port %d\n", relay_port);
             return SR_SUCCESS;
         } else {
             return ret;
@@ -340,18 +333,19 @@ try_port (u_short port, char *if_name, char *relay_ip)
     struct sockaddr_in local;
 
     m_listensock = socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
-    if (m_listensock == SOCKET_ERROR)
+    if (m_listensock == SOCKET_ERROR) {
+	debug_printf ("try_port(%d) failed socket() call\n", port);
         return SR_ERROR_SOCK_BASE;
+    }
     make_nonblocking(m_listensock);
 
     if ('\0' == *relay_ip) {
-	    if (read_interface(if_name,&local.sin_addr.s_addr) != 0)
-		local.sin_addr.s_addr = htonl(INADDR_ANY);
+	if (read_interface(if_name,&local.sin_addr.s_addr) != 0)
+	    local.sin_addr.s_addr = htonl(INADDR_ANY);
     } else {
-	    he = gethostbyname(relay_ip);
-	    memcpy(&local.sin_addr, he->h_addr_list[0], he->h_length);
+	he = gethostbyname(relay_ip);
+	memcpy(&local.sin_addr, he->h_addr_list[0], he->h_length);
     }
-	    
 
     local.sin_family = AF_INET;
     local.sin_port = htons(port);
@@ -366,6 +360,7 @@ try_port (u_short port, char *if_name, char *relay_ip)
                         
     if (bind(m_listensock, (struct sockaddr *)&local, sizeof(local)) == SOCKET_ERROR)
     {
+	debug_printf ("try_port(%d) failed bind() call\n", port);
         closesocket(m_listensock);
         m_listensock = SOCKET_ERROR;
         return SR_ERROR_CANT_BIND_ON_PORT;
@@ -373,34 +368,34 @@ try_port (u_short port, char *if_name, char *relay_ip)
         
     if (listen(m_listensock, 1) == SOCKET_ERROR)
     {
+	debug_printf ("try_port(%d) failed listen() call\n", port);
         closesocket(m_listensock);
         m_listensock = SOCKET_ERROR;
         return SR_ERROR_SOCK_BASE;
     }
 
+    debug_printf ("try_port(%d) succeeded\n", port);
     return SR_SUCCESS;
 }
 
-
 void
 relaylib_shutdown ()
 {
     debug_printf("relaylib_shutdown:start\n");
-    if (!relaylib_isrunning())
-    {
+    if (!relaylib_isrunning()) {
         debug_printf("***relaylib_shutdown:return\n");
         return;
     }
     m_running = FALSE;
     threadlib_signal_sem(&m_sem_not_connected);
-    if (closesocket(m_listensock) == SOCKET_ERROR)
-    {   
+    if (closesocket(m_listensock) == SOCKET_ERROR) {   
         // JCBUG, what can we do?
     }
     m_listensock = SOCKET_ERROR;                // Accept thread will watch for this and not try to accept anymore
     memset(m_http_header, 0, MAX_HEADER_LEN);
     debug_printf("waiting for relay close\n");
     threadlib_waitforclose(&m_hthread);
+    threadlib_waitforclose(&m_hthread2);
     destroy_all_hostsocks();
     threadlib_destroy_sem(&m_sem_not_connected);
     m_initdone = FALSE;
@@ -415,10 +410,10 @@ relaylib_start ()
 
     m_running = TRUE;
     // Spawn on a thread so it's non-blocking
-    if ((ret = threadlib_beginthread(&m_hthread, thread_accept)) != SR_SUCCESS)
+    if ((ret = threadlib_beginthread(&m_hthread, thread_accept, 0)) != SR_SUCCESS)
         return ret;
 
-    if ((ret = threadlib_beginthread(&m_hthread2, thread_send)) != SR_SUCCESS)
+    if ((ret = threadlib_beginthread(&m_hthread2, thread_send, 0)) != SR_SUCCESS)
         return ret;
 
     return SR_SUCCESS;
@@ -436,9 +431,9 @@ thread_accept (void *notused)
     int icy_metadata;
     char* client_http_header;
 
-    debug_printf("***thread_accept:start\n");
+    debug_printf("thread_accept:start\n");
 
-    while(m_running)
+    while (m_running)
     {
         fd_set fds;
         struct timeval tv;
@@ -447,9 +442,12 @@ thread_accept (void *notused)
 	//   have a connection active
         // when a connection gets dropped, or when streamripper shuts down
         //   this event will get signaled
+	debug_printf("thread_accept:waiting on m_sem_not_connected\n");
         threadlib_waitfor_sem (&m_sem_not_connected);
-        if (!m_running)
+        if (!m_running) {
+	    debug_printf("thread_accept:exit (no longer m_running)\n");
             break;
+	}
 
         // Poll once per second, instead of blocking forever in 
 	// accept(), so that we can regain control if relaylib_shutdown() 
@@ -460,7 +458,9 @@ thread_accept (void *notused)
             FD_SET (m_listensock, &fds);
             tv.tv_sec = 1;
             tv.tv_usec = 0;
+	    debug_printf("thread_accept:calling select()\n");
             ret = select (m_listensock + 1, &fds, NULL, NULL, &tv);
+	    debug_printf("thread_accept:select() returned\n");
             if (ret == 1) {
                 unsigned long num_connected;
                 /* If connections are full, do nothing.  Note that 
@@ -472,6 +472,7 @@ thread_accept (void *notused)
                     continue;
                 }
                 /* Check for connections */
+		debug_printf ("Calling accept()\n");
                 newsock = accept (m_listensock, (struct sockaddr *)&client, &iAddrSize);
                 if (newsock != SOCKET_ERROR) {
                     // Got successful accept
