@@ -26,7 +26,7 @@
 #include "srtypes.h"
 #include "cbuf2.h"
 #include "findsep.h"
-#include "util.h"
+#include "mchar.h"
 #include "parse.h"
 #include "rip_manager.h"
 #include "ripstream.h"
@@ -41,9 +41,9 @@
  * Private functions
  *****************************************************************************/
 static error_code find_sep (u_long *pos1, u_long *pos2);
-static error_code end_track_mp3 (u_long pos1, u_long pos2, TRACK_INFO* ti);
 static error_code start_track_mp3 (TRACK_INFO* ti);
-static error_code end_track_ogg (TRACK_INFO* ti);
+static error_code end_track_mp3 (RIP_MANAGER_OPTIONS* rmo, u_long pos1, u_long pos2, TRACK_INFO* ti);
+static error_code end_track_ogg (RIP_MANAGER_OPTIONS* rmo, TRACK_INFO* ti);
 
 static void compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt,
 				  int bitrate, int meta_interval);
@@ -54,8 +54,8 @@ static int ripstream_recvall (char* buffer, int size);
 
 static error_code get_track_from_metadata (int size, char *newtrack);
 static error_code get_stream_data (char *data_buf, char *track_buf);
-static error_code ripstream_rip_ogg (void);
-static error_code ripstream_rip_mp3 (void);
+static error_code ripstream_rip_mp3 (RIP_MANAGER_OPTIONS* rmo);
+static error_code ripstream_rip_ogg (RIP_MANAGER_OPTIONS* rmo);
 
 /*****************************************************************************
  * Private Vars
@@ -69,7 +69,9 @@ static int m_first_time_through;
 static char			m_no_meta_name[MAX_TRACK_LEN] = {'\0'};
 static char			*m_getbuffer = NULL;
 static int			m_find_silence = -1;
-static BOOL			m_addID3tag = TRUE;
+//static BOOL			m_addID3tag = TRUE;
+static BOOL			m_add_id3v1 = TRUE;
+static BOOL			m_add_id3v2 = TRUE;
 static SPLITPOINT_OPTIONS	*m_sp_opt;
 static int			m_bitrate;
 static int			m_http_bitrate;
@@ -103,14 +105,16 @@ typedef struct ID3V1st
         char    year[4];
         char    comment[30];
         char    genre;
-} ID3Tag;
+} ID3V1Tag;
 
+#if defined (commentout)
 typedef struct ID3V2headst {
 	char	tag[3];
 	int	version;
 	char	flags;
 	int	size;
 } ID3V2head;
+#endif
 
 typedef struct ID3V2framest {
 	char	id[4];
@@ -129,7 +133,8 @@ ripstream_init (HSOCKET sock,
 		int bitrate, 
 		int meta_interval, 
 		int content_type, 
-		BOOL addID3tag,
+		BOOL add_id3v1,
+		BOOL add_id3v2,
 		External_Process* ep)
 {
     if (!sp_opt || !no_meta_name) {
@@ -142,7 +147,8 @@ ripstream_init (HSOCKET sock,
     m_timeout = timeout;
     m_sp_opt = sp_opt;
     m_track_count = 0;
-    m_addID3tag = addID3tag;
+    m_add_id3v1 = add_id3v1;
+    m_add_id3v2 = add_id3v2;
     strcpy(m_no_meta_name, no_meta_name);
     m_drop_count = drop_count;
     m_http_bitrate = bitrate;
@@ -187,7 +193,8 @@ ripstream_destroy()
 
     m_no_meta_name[0] = '\0';
     m_track_count = 0;
-    m_addID3tag = TRUE;
+    m_add_id3v1 = TRUE;
+    m_add_id3v2 = TRUE;
 
     /* from ripshout */
     m_buffersize = 0;
@@ -209,20 +216,20 @@ is_track_changed()
 static void
 format_track_info (TRACK_INFO* ti, char* tag)
 {
-    debug_printf ("----- TRACK_INFO %s\n"
-	"HAVETI: %d\n"
-	"RAW_MD: %s\n"
-	"ARTIST: %s\n"
-	"TITLE:  %s\n"
-	"ALBUM:  %s\n"
-	"SAVE:   %d\n",
-	tag,
-        ti->have_track_info,
-	ti->raw_metadata,
-	ti->artist,
-	ti->title,
-	ti->album,
-	ti->save_track);
+    debug_mprintf (m_("----- TRACK_INFO ") m_s m_("\n")
+		   m_("HAVETI: %d\n")
+		   m_("RAW_MD: ") m_s m_("\n")
+		   m_("ARTIST: ") m_S m_("\n")
+		   m_("TITLE:  ") m_S m_("\n")
+		   m_("ALBUM:  ") m_S m_("\n")
+		   m_("SAVE:   %d\n"),
+		   tag,
+		   ti->have_track_info,
+		   ti->raw_metadata,
+		   ti->artist,
+		   ti->title,
+		   ti->album,
+		   ti->save_track);
 }
 
 static void
@@ -243,26 +250,26 @@ copy_track_info (TRACK_INFO* dest, TRACK_INFO* src)
 {
     dest->have_track_info = src->have_track_info;
     strcpy (dest->raw_metadata, src->raw_metadata);
-    strcpy (dest->artist, src->artist);
-    strcpy (dest->title, src->title);
-    strcpy (dest->album, src->album);
+    mstrcpy (dest->artist, src->artist);
+    mstrcpy (dest->title, src->title);
+    mstrcpy (dest->album, src->album);
     strcpy (dest->composed_metadata, src->composed_metadata);
     dest->save_track = src->save_track;
 }
 
 /**** The main loop for ripping ****/
 error_code
-ripstream_rip (void)
+ripstream_rip (RIP_MANAGER_OPTIONS* rmo)
 {
     if (m_content_type == CONTENT_TYPE_OGG) {
-	return ripstream_rip_ogg ();
+	return ripstream_rip_ogg (rmo);
     } else {
-	return ripstream_rip_mp3 ();
+	return ripstream_rip_mp3 (rmo);
     }
 }
 
 static error_code
-ripstream_rip_ogg (void)
+ripstream_rip_ogg (RIP_MANAGER_OPTIONS* rmo)
 {
     int ret;
     int real_ret = SR_SUCCESS;
@@ -325,7 +332,7 @@ ripstream_rip_ogg (void)
 		return ret;
 	    }
 	    if (got_eos) {
-		end_track_ogg (&m_old_track);
+		end_track_ogg (rmo, &m_old_track);
 		have_track = 0;
 		break;
 	    }
@@ -364,7 +371,7 @@ ripstream_rip_ogg (void)
 }
 
 static error_code
-ripstream_rip_mp3 (void)
+ripstream_rip_mp3 (RIP_MANAGER_OPTIONS* rmo)
 {
     int ret;
     int real_ret = SR_SUCCESS;
@@ -437,8 +444,6 @@ ripstream_rip_mp3 (void)
 	if (!m_current_track.have_track_info) {
 	    strcpy (m_current_track.raw_metadata, m_no_meta_name);
 	}
-	debug_printf ("rip_manager_start_track: ti=%s\n", 
-		      m_current_track.title);
 	ret = rip_manager_start_track (&m_current_track, m_track_count);
 	if (ret != SR_SUCCESS) {
 	    debug_printf ("rip_manager_start_track failed(#1): %d\n",ret);
@@ -449,7 +454,8 @@ ripstream_rip_mp3 (void)
     }
 
     /* Check for track change. */
-    debug_printf ("m_current_track.have_track_info = %d\n", m_current_track.have_track_info);
+    debug_printf ("m_current_track.have_track_info = %d\n", 
+		  m_current_track.have_track_info);
     if (m_current_track.have_track_info && is_track_changed()) {
 	/* Set m_find_silence equal to the number of additional blocks 
 	   needed until we can do silence separation. */
@@ -486,7 +492,7 @@ ripstream_rip_mp3 (void)
 	}
 
 	/* Write out previous track */
-	ret = end_track_mp3 (pos1, pos2, &m_old_track);
+	ret = end_track_mp3 (rmo, pos1, pos2, &m_old_track);
 	if (ret != SR_SUCCESS)
 	    real_ret = ret;
 	m_cue_sheet_bytes += pos2;
@@ -561,11 +567,11 @@ find_sep (u_long *pos1, u_long *pos2)
 	debug_printf ("PEEK OK\n");
 
 	/* Find silence point */
-	/* GCS FIX: This doesn't yet consider sw vs rw */
-	ret = findsep_silence (buf, bufsize,
+	ret = findsep_silence (buf, 
+			       bufsize, 
 			       m_rw_start_to_sw_start,
 			       m_sp_opt->xs_search_window_1 
-			       + m_sp_opt->xs_search_window_2,
+				+ m_sp_opt->xs_search_window_2,
 			       m_sp_opt->xs_silence_length,
 			       m_sp_opt->xs_padding_1,
 			       m_sp_opt->xs_padding_2,
@@ -578,7 +584,7 @@ find_sep (u_long *pos1, u_long *pos2)
 }
 
 static error_code
-end_track_mp3 (u_long pos1, u_long pos2, TRACK_INFO* ti)
+end_track_mp3 (RIP_MANAGER_OPTIONS* rmo, u_long pos1, u_long pos2, TRACK_INFO* ti)
 {
     // pos1 is end of prev track
     // pos2 is beginning of next track
@@ -609,15 +615,18 @@ end_track_mp3 (u_long pos1, u_long pos2, TRACK_INFO* ti)
 	goto BAIL;
 
     /* This is id3v1 */
-    if (m_addID3tag) {
-	ID3Tag id3;
-	memset(&id3, '\000',sizeof(id3));
-	strncpy(id3.tag, "TAG", strlen("TAG"));
-	strncpy(id3.artist, ti->artist, sizeof(id3.artist));
-	strncpy(id3.songtitle, ti->title, sizeof(id3.songtitle));
-	strncpy(id3.album, ti->album, sizeof(id3.album));
+    if (m_add_id3v1) {
+	ID3V1Tag id3;
+	memset (&id3, '\000',sizeof(id3));
+	strncpy (id3.tag, "TAG", strlen("TAG"));
+	string_from_mstring (id3.artist, sizeof(id3.artist), 
+			     ti->artist, CODESET_ID3);
+	string_from_mstring (id3.songtitle, sizeof(id3.songtitle), 
+			     ti->title, CODESET_ID3);
+	string_from_mstring (id3.album, sizeof(id3.album),
+			     ti->album, CODESET_ID3);
 	id3.genre = (char) 0xFF; // see http://www.id3.org/id3v2.3.0.html#secA
-	ret = rip_manager_put_data((char *)&id3, sizeof(id3));
+	ret = rip_manager_put_data ((char *)&id3, sizeof(id3));
 	if (ret != SR_SUCCESS) {
 	    goto BAIL;
 	}
@@ -628,7 +637,7 @@ end_track_mp3 (u_long pos1, u_long pos2, TRACK_INFO* ti)
     debug_printf("Current track number %d (skipping if %d or less)\n", 
 		 m_track_count, m_drop_count);
     if (m_track_count > m_drop_count)
-	if ((ret = rip_manager_end_track(ti)) != SR_SUCCESS)
+	if ((ret = rip_manager_end_track (rmo, ti)) != SR_SUCCESS)
 	    goto BAIL;
 
  BAIL:
@@ -642,14 +651,6 @@ start_track_mp3 (TRACK_INFO* ti)
 #define HEADER_SIZE 1600
     int ret;
     int i;
-    ID3V2frame id3v2frame1;
-    ID3V2frame id3v2frame2;
-    char comment[1024] = "Ripped with Streamripper";
-    char bigbuf[HEADER_SIZE] = "";
-    int	sent = 0;
-    char header1[6] = "ID3\x03\0\0";
-    int header_size = HEADER_SIZE;
-    unsigned long int framesize = 0;
     unsigned int secs;
 
     debug_printf ("calling rip_manager_start_track(#2)\n");
@@ -666,10 +667,18 @@ start_track_mp3 (TRACK_INFO* ti)
         return ret;
 
     /* Oddsock's ID3 stuff, (oddsock@oddsock.org) */
-    if (m_addID3tag) {
+    if (m_add_id3v2) {
+	int rc;
+	char bigbuf[HEADER_SIZE] = "";
+	int header_size = HEADER_SIZE;
+	char header1[6] = "ID3\x03\0\0";
+	ID3V2frame id3v2frame;
+	char comment[1024] = "Ripped with Streamripper";
+	unsigned long int framesize = 0;
+	int sent = 0;
+	int id3_charset;
+
 	memset(bigbuf, '\000', sizeof(bigbuf));
-	memset(&id3v2frame1, '\000', sizeof(id3v2frame1));
-	memset(&id3v2frame2, '\000', sizeof(id3v2frame2));
 
 	/* Write header */
 	ret = rip_manager_put_data(header1, 6);
@@ -680,72 +689,93 @@ start_track_mp3 (TRACK_INFO* ti)
 	    if (ret != SR_SUCCESS) return ret;
 	}
 
-	// Write ID3V2 frame1 with data
-	strncpy(id3v2frame1.id, "TPE1", 4);
-	framesize = htonl(strlen(ti->artist)+1);
-	ret = rip_manager_put_data((char *)&(id3v2frame1.id), 4);
+	/* ID3 V2.3 is only defined for ISO-8859-1 and UCS-2
+	   If user specifies another codeset, we will use it, and 
+	   report ISO-8859-1 in the encoding field */
+	id3_charset = is_id3_unicode();
+
+	/* Write ID3V2 TPE1 frame (Lead performer) */
+	memset(&id3v2frame, '\000', sizeof(id3v2frame));
+	strncpy(id3v2frame.id, "TPE1", 4);
+	id3v2frame.pad[2] = id3_charset;
+	rc = string_from_mstring (bigbuf, HEADER_SIZE, ti->artist, 
+				  CODESET_ID3);
+	framesize = htonl (rc+1);
+	ret = rip_manager_put_data ((char *)&(id3v2frame.id), 4);
+	if (ret != SR_SUCCESS) return ret;
+	sent += 4;
+	ret = rip_manager_put_data ((char *)&(framesize), sizeof(framesize));
+	if (ret != SR_SUCCESS) return ret;
+	sent += sizeof(framesize);
+	ret = rip_manager_put_data ((char *)&(id3v2frame.pad), 3);
+	if (ret != SR_SUCCESS) return ret;
+	sent += 3;
+	ret = rip_manager_put_data (bigbuf, rc);
+	if (ret != SR_SUCCESS) return ret;
+	sent += rc;
+
+	/* Write ID3V2 TIT2 frame (Title) */
+	memset(&id3v2frame, '\000', sizeof(id3v2frame));
+	strncpy(id3v2frame.id, "TIT2", 4);
+	id3v2frame.pad[2] = id3_charset;
+	rc = string_from_mstring (bigbuf, HEADER_SIZE, ti->title, 
+				  CODESET_ID3);
+	framesize = htonl (rc+1);
+	ret = rip_manager_put_data((char *)&(id3v2frame.id), 4);
 	if (ret != SR_SUCCESS) return ret;
 	sent += 4;
 	ret = rip_manager_put_data((char *)&(framesize), sizeof(framesize));
 	if (ret != SR_SUCCESS) return ret;
 	sent += sizeof(framesize);
-	ret = rip_manager_put_data((char *)&(id3v2frame1.pad), 3);
+	ret = rip_manager_put_data((char *)&(id3v2frame.pad), 3);
 	if (ret != SR_SUCCESS) return ret;
 	sent += 3;
-	ret = rip_manager_put_data(ti->artist, strlen(ti->artist));
+	ret = rip_manager_put_data (bigbuf, rc);
 	if (ret != SR_SUCCESS) return ret;
-	sent += strlen(ti->artist);
+	sent += rc;
 
-	// Write ID3V2 frame2 with data
-	strncpy(id3v2frame2.id, "TIT2", 4);
-	framesize = htonl(strlen(ti->title)+1);
-	ret = rip_manager_put_data((char *)&(id3v2frame2.id), 4);
-	if (ret != SR_SUCCESS) return ret;
-	sent += 4;
-	ret = rip_manager_put_data((char *)&(framesize), sizeof(framesize));
-	if (ret != SR_SUCCESS) return ret;
-	sent += sizeof(framesize);
-	ret = rip_manager_put_data((char *)&(id3v2frame2.pad), 3);
-	if (ret != SR_SUCCESS) return ret;
-	sent += 3;
-	ret = rip_manager_put_data(ti->title, strlen(ti->title));
-	if (ret != SR_SUCCESS) return ret;
-	sent += strlen(ti->title);
-
-	// Write ID3V2 frame2 with data
-	strncpy(id3v2frame2.id, "TENC", 4);
+	/* Write ID3V2 TENC frame (Encoded by) */
+	memset(&id3v2frame, '\000', sizeof(id3v2frame));
+	strncpy(id3v2frame.id, "TENC", 4);
 	framesize = htonl(strlen(comment)+1);
-	ret = rip_manager_put_data((char *)&(id3v2frame2.id), 4);
+	ret = rip_manager_put_data((char *)&(id3v2frame.id), 4);
 	if (ret != SR_SUCCESS) return ret;
 	sent += 4;
 	ret = rip_manager_put_data((char *)&(framesize), sizeof(framesize));
 	if (ret != SR_SUCCESS) return ret;
 	sent += sizeof(framesize);
-	ret = rip_manager_put_data((char *)&(id3v2frame2.pad), 3);
+	ret = rip_manager_put_data((char *)&(id3v2frame.pad), 3);
 	if (ret != SR_SUCCESS) return ret;
 	sent += 3;
 	ret = rip_manager_put_data(comment, strlen(comment));
 	if (ret != SR_SUCCESS) return ret;
-	sent += strlen(comment);
+	sent += strlen (comment);
 
-	// Write ID3V2 frame2 with data
-	memset(&id3v2frame2, '\000', sizeof(id3v2frame2));
-	strncpy(id3v2frame2.id, "TALB", 4);
-	framesize = htonl(strlen(ti->album)+1);
-	ret = rip_manager_put_data((char *)&(id3v2frame2.id), 4);
+	/* Write ID3V2 TALB frame (Album) */
+	memset(&id3v2frame, '\000', sizeof(id3v2frame));
+	strncpy(id3v2frame.id, "TALB", 4);
+	id3v2frame.pad[2] = id3_charset;
+	rc = string_from_mstring (bigbuf, HEADER_SIZE, ti->album, 
+				  CODESET_ID3);
+	framesize = htonl (rc+1);
+	ret = rip_manager_put_data((char *)&(id3v2frame.id), 4);
 	if (ret != SR_SUCCESS) return ret;
 	sent += 4;
 	ret = rip_manager_put_data((char *)&(framesize), sizeof(framesize));
 	if (ret != SR_SUCCESS) return ret;
 	sent += sizeof(framesize);
-	ret = rip_manager_put_data((char *)&(id3v2frame2.pad), 3);
+	ret = rip_manager_put_data((char *)&(id3v2frame.pad), 3);
 	if (ret != SR_SUCCESS) return ret;
 	sent += 3;
-	ret = rip_manager_put_data(ti->album, strlen(ti->album));
+	ret = rip_manager_put_data (bigbuf, rc);
 	if (ret != SR_SUCCESS) return ret;
-	sent += strlen(ti->album);
+	sent += rc;
 
-	ret = rip_manager_put_data(bigbuf, 1600-sent);
+	/* Zero out padding */
+	memset(bigbuf, '\000', sizeof(bigbuf));
+
+	/* Pad up to header_size */
+	ret = rip_manager_put_data(bigbuf, HEADER_SIZE-sent);
 	if (ret != SR_SUCCESS) return ret;
     }
     m_track_count ++;
@@ -757,13 +787,13 @@ start_track_mp3 (TRACK_INFO* ti)
 // Only save this track if we've skipped over enough cruft 
 // at the beginning of the stream
 static error_code
-end_track_ogg (TRACK_INFO* ti)
+end_track_ogg (RIP_MANAGER_OPTIONS* rmo, TRACK_INFO* ti)
 {
     error_code ret;
     debug_printf ("Current track number %d (skipping if %d or less)\n", 
 		  m_track_count, m_drop_count);
     if (m_track_count > m_drop_count) {
-	ret = rip_manager_end_track (ti);
+	ret = rip_manager_end_track (rmo, ti);
     } else {
 	ret = SR_SUCCESS;
     }
@@ -771,6 +801,7 @@ end_track_ogg (TRACK_INFO* ti)
     return ret;
 }
 
+#if defined (commentout)
 /* GCS: This converts either positive or negative ms to blocks,
    and must work for rounding up and rounding down */
 static int
@@ -791,6 +822,7 @@ ms_to_blocks (int ms, int bitrate, int round_up)
     }
     return blocks;
 }
+#endif
 
 /* Simpler routine, rounded toward zero */
 static int
@@ -812,12 +844,62 @@ bytes_to_secs (unsigned int bytes)
     return secs;
 }
 
+/* --------------------------------------------------------------------------
+   Buffering for silence splitting & padding
+
+   We may flush the circular buffer as soon as anything that 
+   needs to go into the next song has passed.   For simplicity, 
+   we also buffer up to the latest point that can go into the 
+   next song.  This is called the "required window."
+
+   The entire required window is decoded, even though 
+   we don't need the volume.  We simply mark the frame boundaries
+   so we don't chop any frames.
+
+   The circular buffer is a bit bigger than the required window, 
+   because it contains an integer number of blocks which fall 
+
+   Some abbreviations:
+     mic      meta inf change
+     cb	      cbuf2, aka circular buffer
+     rw	      required window
+     sw	      search window
+
+   This is the complete picture:
+
+    A---------A---------A----+----B---------B     meta intervals
+
+                                  /mic            meta-inf change (A to B)
+                             +--->|
+                             /mi                  meta-inf point
+                             |
+                   |---+-----+------+---|         search window
+                       |            |   
+                       |            |   
+                   |---+---|    |---+---|         silence length
+                       |            |
+         |-------------|            |-----|
+              prepad                postpad
+
+         |--------------------------------|       required window
+
+    |---------------------------------------|     cbuf
+
+
+                   |<-------------+               mic_to_sw_start
+                                  +---->|         mic_to_sw_end
+         |<-----------------------+               mic_to_rw_start
+                                  +------>|       mic_to_rw_end
+    |<----------------------------+               mic_to_cb_start
+                                  +-------->|     mic_to_cb_end
+
+   ------------------------------------------------------------------------*/
 static void
 compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate, 
 		      int meta_interval)
 {
     long sws, sl;
-    long mic_to_mi;
+    long mi_to_mic;
     long prepad, postpad;
     long offset;
     long mic_to_sw_start, mic_to_sw_end;
@@ -835,10 +917,10 @@ compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate,
     debug_printf ("bitrate = %d, meta_inf = %d\n", bitrate, meta_interval);
     debug_printf ("---------------------------------------------------\n");
     
-    /* mic_to_mi is the "half of a meta-inf" from the meta inf 
+    /* mi_to_mic is the "half of a meta-inf" from the meta inf 
        change to the previous (non-changed) meta inf */
-    mic_to_mi = meta_interval / 2;
-    debug_printf ("mic_to_mi: %d\n", mic_to_mi);
+    mi_to_mic = meta_interval / 2;
+    debug_printf ("mi_to_mic: %d\n", mi_to_mic);
 
     /* compute the search window size (sws) */
     sws = ms_to_bytes (sp_opt->xs_search_window_1, bitrate) 
@@ -859,9 +941,9 @@ compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate,
     debug_printf ("offset: %d\n", offset);
 
     /* compute interval from mi to search window */
-    mic_to_sw_start = - mic_to_mi + offset 
+    mic_to_sw_start = - mi_to_mic + offset 
 	    - ms_to_bytes(sp_opt->xs_search_window_1,bitrate);
-    mic_to_sw_end = - mic_to_mi + offset 
+    mic_to_sw_end = - mi_to_mic + offset 
 	    + ms_to_bytes(sp_opt->xs_search_window_2,bitrate);
     debug_printf ("mic_to_sw_start: %d\n", mic_to_sw_start);
     debug_printf ("mic_to_sw_end: %d\n", mic_to_sw_end);
@@ -887,8 +969,6 @@ compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate,
     if (mic_to_cb_end < 0) {
 	mic_to_cb_end = 0;
     }
-    debug_printf ("mic_to_cb_start: %d\n", mic_to_cb_start);
-    debug_printf ("mic_to_cb_end: %d\n", mic_to_cb_end);
 
     /* Convert to chunks & compute cbuf size */
     mic_to_cb_end = (mic_to_cb_end + (meta_interval-1)) / meta_interval;
@@ -898,6 +978,8 @@ compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate,
     if (m_cbuf2_size < 3) {
 	m_cbuf2_size = 3;
     }
+    debug_printf ("mic_to_cb_start: %d\n", mic_to_cb_start * meta_interval);
+    debug_printf ("mic_to_cb_end: %d\n", mic_to_cb_end * meta_interval);
     debug_printf ("CBUF2 (BLOCKS): %d:%d -> %d\n", mic_to_cb_start,
 		  mic_to_cb_end, m_cbuf2_size);
 
@@ -905,8 +987,7 @@ compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate,
     m_mic_to_cb_end = mic_to_cb_end;
     m_rw_start_to_cb_end = mic_to_cb_end * meta_interval - mic_to_rw_start;
     m_rw_end_to_cb_end = mic_to_cb_end * meta_interval - mic_to_rw_end;
-    m_rw_start_to_sw_start = sp_opt->xs_padding_1 
-	    - (sp_opt->xs_search_window_1 + sp_opt->xs_search_window_2);
+    m_rw_start_to_sw_start = sp_opt->xs_padding_1 - sp_opt->xs_silence_length / 2;
     if (m_rw_start_to_sw_start < 0) {
 	m_rw_start_to_sw_start = 0;
     }
