@@ -68,6 +68,9 @@ typedef int error_code;
 #define MAX_AUTH_LEN            255
 //#define MAX_DROPSTRING_LEN      255
 
+#define MAX_METADATA_LEN (127*16)
+
+
 #ifdef WIN32
   #ifndef _WINSOCKAPI_
     #define __DEFINE_TYPES__
@@ -136,28 +139,48 @@ typedef struct SPLITPOINT_OPTIONSst
  * CODESET_OPTIONS are the options used to decide how to parse
  * and convert the metadata
  */
+#define MAX_CODESET_STRING 128
 typedef struct CODESET_OPTIONSst
 {
-    char* codeset;
+    char codeset_locale[MAX_CODESET_STRING];
+    char codeset_filesys[MAX_CODESET_STRING];
+    char codeset_id3[MAX_CODESET_STRING];
+    char codeset_metadata[MAX_CODESET_STRING];
+    char codeset_relay[MAX_CODESET_STRING];
 } CODESET_OPTIONS;
 
+/* 
+ * Various CODESET types
+ */
+#define CODESET_UTF8          1
+#define CODESET_LOCALE        2
+#define CODESET_FILESYS       3
+#define CODESET_ID3           4
+#define CODESET_METADATA      5
+#define CODESET_RELAY         6
+
+/* 
+ * Wide character support
+ */
+#if HAVE_WCHAR_SUPPORT
+typedef wchar_t mchar;
+#else
+typedef char mchar;
+#endif
+
 /* 
  * TRACK_INFO is the parsed metadata
  */
-#if HAVE_WCHAR_SUPPORT
-typedef char mchar;
-#else
-typedef wchar_t mchar;
-#endif
 typedef struct TRACK_INFOst
 {
     int have_track_info;
     char raw_metadata[MAX_TRACK_LEN];
-    char artist[MAX_TRACK_LEN];
-    char title[MAX_TRACK_LEN];
-    char album[MAX_TRACK_LEN];
-    char track[MAX_TRACK_LEN];
-    char composed_metadata[MAX_TRACK_LEN];
+    mchar w_raw_metadata[MAX_TRACK_LEN];
+    mchar artist[MAX_TRACK_LEN];
+    mchar title[MAX_TRACK_LEN];
+    mchar album[MAX_TRACK_LEN];
+    mchar track[MAX_TRACK_LEN];
+    char composed_metadata[MAX_METADATA_LEN+1];      /* For relay stream */
     BOOL save_track;
 } TRACK_INFO;
 
