@@ -39,6 +39,9 @@ char *alloca ();
 #endif
 #endif /* TRE_USE_ALLOCA */
 
+#define __USE_STRING_INLINES
+#undef __NO_INLINE__
+
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
@@ -220,7 +223,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
   tre_char_t prev_c = 0, next_c = 0;
   const char *str_byte = string;
   int pos = -1;
-  int pos_add_next = 1;
+  unsigned int pos_add_next = 1;
 #ifdef TRE_WCHAR
   const wchar_t *str_wide = string;
 #ifdef TRE_MBSTATE
@@ -367,6 +370,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 		  /* Compute tag values after this transition. */
 		  for (i = 0; i < num_tags; i++)
 		    reach_next[id].tags[i] = -1;
+
 		  if (trans->tags)
 		    for (i = 0; trans->tags[i] >= 0; i++)
 		      if (trans->tags[i] < num_tags)
@@ -402,7 +406,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	 give a better match than what we already have. */
       for (id = 0; id < tnfa->num_states; id++)
 	{
-	  int depth, j;
+	  int depth;
 	  int cost, cost0;
 
 	  if (reach[id].pos != prev_pos)
@@ -460,9 +464,9 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	  reach_next[id].depth = reach[id].depth;
 
 	  /* Set the costs after this transition. */
-	  for (i = 0; i <= depth; i++)
-	    for (j = 0; j < TRE_M_LAST; j++)
-	      reach_next[id].costs[i][j] = reach[id].costs[i][j];
+	  memcpy(reach_next[id].costs, reach[id].costs,
+		 sizeof(reach_next[id].costs[0][0])
+		 * TRE_M_LAST * (depth + 1));
 	  reach_next[id].costs[depth][TRE_M_COST] = cost;
 	  reach_next[id].costs[depth][TRE_M_NUM_INS]++;
 	  reach_next[id].costs[depth][TRE_M_NUM_ERR]++;
@@ -502,7 +506,6 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	    tre_tnfa_approx_reach_t *reach_p;
 	    int id;
 	    int depth;
-	    int j;
 	    int cost, cost0;
 	    tre_tnfa_transition_t *trans;
 
@@ -569,9 +572,10 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 		    && (cost0 > reach_next[dest_id].costs[0][TRE_M_COST]
 			|| (cost0 == reach_next[dest_id].costs[0][TRE_M_COST]
 			    && (!match_tags
-				|| tre_tag_order(num_tags, tnfa->tag_directions,
-						 reach_next[dest_id].tags,
-						 tmp_tags)))))
+				|| !tre_tag_order(num_tags,
+						  tnfa->tag_directions,
+						  tmp_tags,
+						  reach_next[dest_id].tags)))))
 		  {
 		    DPRINT(("lose, cost0 %d, have %d\n",
 			    cost0, reach_next[dest_id].costs[0][TRE_M_COST]));
@@ -591,9 +595,10 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 				 default_params);
 
 		reach_next[dest_id].depth = reach_p->depth;
-		for (i = 0; i <= depth; i++)
-		  for (j = 0; j < TRE_M_LAST; j++)
-		    reach_next[dest_id].costs[i][j] = reach_p->costs[i][j];
+		memcpy(&reach_next[dest_id].costs,
+		       reach_p->costs,
+		       sizeof(reach_p->costs[0][0])
+		       * TRE_M_LAST * (depth + 1));
 		reach_next[dest_id].costs[depth][TRE_M_COST] = cost;
 		reach_next[dest_id].costs[depth][TRE_M_NUM_DEL]++;
 		reach_next[dest_id].costs[depth][TRE_M_NUM_ERR]++;
@@ -614,8 +619,8 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 		    DPRINT(("	 setting new match at %d, cost %d\n",
 			    pos, cost0));
 		    match_eo = pos;
-		    for (i = 0; i < TRE_M_LAST; i++)
-		      match_costs[i] = reach_next[dest_id].costs[0][i];
+		    memcpy(match_costs, reach_next[dest_id].costs[0],
+			   sizeof(match_costs[0]) * TRE_M_LAST);
 		    for (i = 0; i < num_tags; i++)
 		      match_tags[i] = tmp_tags[i];
 		  }
@@ -671,7 +676,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	  for (trans = reach[id].state; trans->state; trans++)
 	    {
 	      int dest_id;
-	      int j, depth;
+	      int depth;
 	      int cost, cost0, err;
 
 	      if (trans->assertions
@@ -757,9 +762,9 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	      if (reach_next[dest_id].pos == pos
 		  && (cost0 > reach_next[dest_id].costs[0][TRE_M_COST]
 		      || (cost0 == reach_next[dest_id].costs[0][TRE_M_COST]
-			  && tre_tag_order(num_tags, tnfa->tag_directions,
-					   reach_next[dest_id].tags,
-					   tmp_tags))))
+			  && !tre_tag_order(num_tags, tnfa->tag_directions,
+					    tmp_tags,
+					    reach_next[dest_id].tags))))
 		{
 		  DPRINT(("lose\n"));
 		  continue;
@@ -782,9 +787,10 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 			       default_params);
 
 	      /* Set the costs after this transition. */
-	      for (i = 0; i <= depth; i++)
-		for (j = 0; j < TRE_M_LAST; j++)
-		  reach_next[dest_id].costs[i][j] = reach[id].costs[i][j];
+		memcpy(&reach_next[dest_id].costs,
+		       reach[id].costs,
+		       sizeof(reach[id].costs[0][0])
+		       * TRE_M_LAST * (depth + 1));
 	      reach_next[dest_id].costs[depth][TRE_M_COST] = cost;
 	      reach_next[dest_id].costs[depth][TRE_M_NUM_SUBST] += err;
 	      reach_next[dest_id].costs[depth][TRE_M_NUM_ERR] += err;
